@@ -6,7 +6,7 @@ use XSLoader;
 
 use Exporter 5.57 'import';
 
-our $VERSION     = '0.020';
+our $VERSION     = '0.100';
 our %EXPORT_TAGS = ( 'all' => [qw<
     is_ref
     is_scalarref
@@ -37,7 +37,7 @@ our %EXPORT_TAGS = ( 'all' => [qw<
     is_blessed_formatref
     is_blessed_refref
 >] );
-our @EXPORT      = ();
+
 our @EXPORT_OK   = ( @{ $EXPORT_TAGS{'all'} } );
 
 XSLoader::load('Ref::Util', $VERSION);
@@ -56,7 +56,7 @@ Ref::Util - Utility functions for checking references
 
 =head1 VERSION
 
-0.020
+0.100
 
 =head1 DESCRIPTION
 
@@ -75,41 +75,52 @@ The difference:
 
 =item * Fast
 
-The benchmark:
+Here is a benchmark comparing similar checks.
 
     my $bench = Dumbbench->new(
         target_rel_precision => 0.005,
         initial_runs         => 20,
     );
 
-    my $ref = [];
+    my $amount = 1e7;
+    my $ref    = [];
     $bench->add_instances(
         Dumbbench::Instance::PerlSub->new(
-            name => 'XS',
-            code => sub { Ref::Util::is_arrayref($ref) for(1..1e7) },
+            name => 'Ref::Util::is_plain_arrayref (CustomOP)',
+            code => sub {
+                Ref::Util::is_plain_arrayref($ref) for ( 1 .. $amount )
+            },
         ),
 
         Dumbbench::Instance::PerlSub->new(
-            name => 'reftype',
-            code => sub { reftype($ref) eq 'ARRAY' for(1..1e7) },
+            name => 'ref(), reftype(), !blessed()',
+            code => sub {
+                ref $ref
+                    && Scalar::Util::reftype($ref) eq 'ARRAY'
+                    && !Scalar::Util::blessed($ref)
+                    for ( 1 .. $amount );
+            },
         ),
 
         Dumbbench::Instance::PerlSub->new(
-            name => 'PP',
-            code => sub { ref($ref) eq 'ARRAY' for(1..1e7) },
+            name => 'ref()',
+            code => sub { ref($ref) eq 'ARRAY' for ( 1 .. $amount ) },
         ),
+
+        Dumbbench::Instance::PerlSub->new(
+            name => 'Data::Util::is_array_ref',
+            code => sub { is_array_ref($ref) for ( 1 .. $amount ) },
+        ),
+
     );
 
 The results:
 
-    XS:      Ran 27 iterations (6 outliers).
-    XS:      Rounded run time per iteration: 3.0093e-01 +/- 4.4e-04 (0.1%)
+    ref():                                   6.2449e-01 +/- 3.1e-04 (0.0%)
+    ref(), reftype(), !blessed():            2.3022e+00 +/- 2.5e-03 (0.1%)
+    Ref::Util::is_plain_arrayref (CustomOP): 3.47157e-01 +/- 6.8e-05 (0.0%)
 
-    reftype: Ran 25 iterations (5 outliers).
-    reftype: Rounded run time per iteration: 9.173e-01 +/- 1.2e-03 (0.1%)
-
-    PP:      Ran 26 iterations (6 outliers).
-    PP:      Rounded run time per iteration: 6.1437e-01 +/- 3.4e-04 (0.1%)
+(Rounded run time per iteration)
 
 =item * No comparison against a string constant
 
@@ -126,6 +137,7 @@ string:
 
 B<Note:> In future versions, the idea is to make the default functions
 use the B<plain> variation, which means explicitly non-blessed references.
+
 If you want to explicitly check for B<blessed> references, you should use
 the C<is_blessed_*> functions. There will be an C<is_any_*> variation
 which will act like the current main functions - not caring whether it's
@@ -155,6 +167,19 @@ There is also a family of functions with names like C<is_blessed_hashref>;
 these return true for blessed object instances that are implemented using
 the relevant underlying type.
 
+=item * Supports tied variables and magic
+
+Tied variables (used in L<Readonly>, for example) are supported.
+
+    use Ref::Util qw<is_plain_hashref>;
+    use Readonly;
+
+    Readonly::Scalar my $rh2 => { a => { b => 2 } };
+    is_plain_hashref($rh2); # success
+
+L<Ref::Util> added support for this in 0.100. Prior to this version
+the test would fail.
+
 =item * Ignores overloading
 
 These functions ignore overloaded operators and simply check the
@@ -165,41 +190,8 @@ Overloading makes your variables opaque containers and hides away
 B<what> they are and instead require you to figure out B<how> to use
 them. This leads to code that has to test different abilities (in
 C<eval>, so it doesn't crash) and to interfaces that get around what
-a person thought you would do with a variable. Ugh. Double Ugh.
-For this reason they are not supported.
-
-This is also not duck-typing, as at least one person suggested. Duck
-typing provides a method that *works* and has different
-implementations. The difference is that here we have different methods
-(stringification, array dereferencing, hash dereferencing, callbacks,
-greater-than comparsion, etc.) which have to be tested each
-individually. This is the B<opposite> of duck-typing. Also, in
-duck-typing you can introspect to know what is available, and
-overloading does not lend to that.
-
-Overloading is I<cool, but terribly horrible>. 'Nuff said.
-
-=item * Readonly, tied variables, and magic
-
-Tied variables (used in L<Readonly>, for example) are not supported,
-as they are not references, but regular variables with added magic.
-
-Consider the following:
-
-    use Data::Printer;
-    use Readonly;
-    Readonly::Scalar my $rh2 => { a => { b => 2 } };
-    p $rh2->{a};
-
-    # result:
-    # "HASH(0x187dcc8)"
-
-This should print a hashref structure with key B<b> and value B<2>,
-but it doesn't. It prints a string. It should have retrieved the
-values but caused stringification instead.
-
-The problem here is that C<< $rh2->{a} >> is not a hashref, but a
-C<PVLV> with magic, so C<is_hashref> will correctly not detect it.
+a person thought you would do with a variable. This would have been
+alright, except there is no clear way of introspecting it.
 
 =item * Ignores subtle types:
 
@@ -235,8 +227,9 @@ an implementation that just calls an XS function - which is still
 faster than the Pure-Perl equivalent.
 
 We might also introduce a Pure-Perl version of everything, allowing
-to install this module where a compiler is not available, making the
-XS parts optional.
+to install and load the faster implementation seamlessly if possible.
+
+This will allow projects to use an optionally-Pure-Perl version.
 
 =head1 EXPORT
 
@@ -273,7 +266,7 @@ reference to another reference is not treated as a scalar reference:
 The rationale for this is two-fold. First, callers that want to decide how
 to handle inputs based on their reference type will usually want to treat a
 ref-ref and a scalar-ref differently. Secondly, this more closely matches
-the behavior of the C<ref> builtin and of L<Scalar::Util/reftype>, which
+the behavior of the C<ref> built-in and of L<Scalar::Util/reftype>, which
 report a ref-ref as C<REF> rather than C<SCALAR>.
 
 =head2 is_arrayref($ref)
@@ -448,6 +441,13 @@ Check for a blessed reference to a reference.
 
 =item * L<Scalar::Util>
 
+=item * L<Data::Util>
+
+A benchmark against L<Data::Util>.
+
+    Ref::Util::is_plain_arrayref: 3.47157e-01 +/- 6.8e-05 (0.0%)
+    Data::Util::is_array_ref:     6.7562e-01 +/- 7.5e-04 (0.1%)
+
 =back
 
 =head1 THANKS
@@ -464,12 +464,20 @@ The following people have been invaluable in their feedback and support.
 
 =item * Mattia Barbon
 
+=item * Zefram
+
+=item * Tony Cook
+
+=item * Sergey Aleynikov
+
 =back
 
 =head1 AUTHORS
 
 =over 4
 
+=item * Aaron Crane
+
 =item * Vikentiy Fesunov
 
 =item * Sawyer X
@@ -478,6 +486,9 @@ The following people have been invaluable in their feedback and support.
 
 =item * p5pclub
 
-=item * Aaron Crane
-
 =back
+
+=head1 LICENSE
+
+This software is made available under the MIT Licence as stated in the
+accompanying LICENSE file.
